/** written by olaf booij
 *  public domain */

#ifndef QRSMURRIE_IMAGE_COMPARE_HPP_INCLUDE
#define QRSMURRIE_IMAGE_COMPARE_HPP_INCLUDE

#include<vector>
#include<qrsmurrie/qrencode_wrapper.hpp>
#include<qrsmurrie/image.hpp>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

namespace qrsmurrie {

  typedef std::vector< bool > MonoImage; // TODO std::vector< bool > is a mistake (http://en.wikipedia.org/wiki/Sequence_container_%28C%2B%2B%29#Specialization_for_bool)

  template< typename MonoImageT, typename QRcodeT >
  unsigned differenceImageQR(const MonoImageT& monoImage, const QRcodeT& qrcode)
  {
    unsigned difference = 0;
    for(unsigned fenoI = 0; fenoI < getNrOfQRValues(qrcode); ++fenoI)
      if(getQRValue(qrcode, fenoI) != monoImage.at(fenoI))
        ++difference;
    return(difference);
  }

  MonoImage generateWhiteImage(const unsigned imageSize)
  {
    return(MonoImage(imageSize, true));
  }
  MonoImage generateBlackImage(const unsigned imageSize)
  {
    return(MonoImage(imageSize, false));
  }
  MonoImage generateRandomImage(const unsigned imageSize)
  { 
    MonoImage image;
    for(unsigned pixelI = 0; pixelI < imageSize; ++pixelI)
      image.push_back(std::rand() % 2 == 1);
    return(image);
  }


  template< typename MonoImageT, typename QRcodeT, typename QRweightT >
  double weightedSimilarityImageQR(const MonoImageT& monoImage, const QRcodeT& qrcode, const QRweightT& qrweight)
  {
    double similarity = 0;
    for(unsigned fenoI = 0; fenoI < getNrOfQRValues(qrcode); ++fenoI)
    {
      if(getQRValue(qrcode, fenoI))
        if(monoImage.at(fenoI))
          similarity += 1.0 - qrweight.at(fenoI);
      if(! getQRValue(qrcode, fenoI))
        if(! monoImage.at(fenoI))
          similarity += qrweight.at(fenoI);
    }
    return(similarity);
  }

}

#endif // QRSMURRIE_IMAGE_COMPARE_HPP_INCLUDE

