/** written by olaf booij
 *  public domain */

#ifndef QRSMURRIE_PERMSORT_HPP_INCLUDE
#define QRSMURRIE_PERMSORT_HPP_INCLUDE

#include<iterator>
#include<algorithm>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

namespace qrsmurrie {

inline
std::vector< std::size_t > counting(const std::size_t size)
{
  std::vector< size_t > permutation(size);
  for(std::size_t permI = 0; permI < size; ++permI)
    permutation.at(permI) = permI;
  return(permutation);
}

template< typename Container >
std::vector< std::size_t > countingContainer(const Container container)
{ return(counting(container.size())); }

template< typename RandomAccessIteratorT = std::vector< double  >::iterator >
struct permLess
{
  const RandomAccessIteratorT& first; 
  permLess(RandomAccessIteratorT& _first) :
    first(_first)
  {}
  template< typename IndexT >
  bool operator()(const IndexT& elementA, const IndexT& elementB) const
  {
    return(*(first + elementA) < *(first + elementB)); // FIXME should be template
  }
};

// sorts the range [first, last> AND returns the permutation used
template< class RandomAccessIteratorT >
std::vector< std::size_t > permSort(RandomAccessIteratorT first, RandomAccessIteratorT last)
{
  typedef std::size_t size_t;
  std::vector< size_t > permutation = counting(std::distance(first, last));
  std::sort(permutation.begin(), permutation.end(), permLess< RandomAccessIteratorT >(first));
  return(permutation);
}


template< typename IndexT, typename RandomAccessIteratorT >
std::vector< typename RandomAccessIteratorT::value_type > applyPerm(const std::vector< IndexT > permutation, const RandomAccessIteratorT first)
{
  std::vector< typename RandomAccessIteratorT::value_type > result;
  foreach(IndexT val, permutation)
    result.push_back(*(first + val));
  return(result);
}

// could probably made faster and not using copy, using a different permLess tool... oh well.
template< typename RandomAccessIteratorT >
std::vector< std::size_t > permSortAndApply(RandomAccessIteratorT first, RandomAccessIteratorT last)
{
  std::vector< std::size_t > permutation = permSort(first, last);
  std::vector< typename RandomAccessIteratorT::value_type > aCopy(first, last);
  for(std::size_t permI = 0; permI < aCopy.size(); ++permI)
    *(first + permI) = aCopy.at(permutation.at(permI));
  return(permutation);
}

// actually I do not like iterators that much:
template< typename ContainerT >
std::vector< std::size_t > permSort(ContainerT container)
{
  return(permSort(container.begin(), container.end()));
}

template< typename IndexT >
std::vector< IndexT > inversePerm(std::vector< IndexT > permutation)
{
  std::reverse(permutation.begin(), permutation.end());
  return(permutation);
}


template< typename IndexT, typename ContainerT >
std::vector< typename ContainerT::value_type > applyPerm2(const std::vector< IndexT > permutation, const ContainerT container)
{
  return(applyPerm(permutation, container.begin()));
}

template< class ContainerT >
std::vector< std::size_t > permSortAndApply(ContainerT& container)
{
  return(permSortAndApply(container.begin(), container.end()));
}

} // namespace qrsmurrie

#endif // QRSMURRIE_PERMSORT_HPP_INCLUDE

